# Generated by roxygen2: do not edit by hand

S3method("==",Matrix4)
S3method("==",Quaternion)
S3method("==",Vector3)
S3method("[",Matrix4)
S3method("[",Quaternion)
S3method("[",Vector3)
S3method(as.matrix,Matrix4)
S3method(as.matrix,Vector3)
S3method(baseline_array,array)
S3method(collapse,array)
S3method(dim,Matrix4)
S3method(dim,Vector3)
S3method(format,"ravetools-design_filter")
S3method(format,"ravetools-freqz2")
S3method(format,"ravetools-printable")
S3method(format,"ravetools-test_filter")
S3method(format,"summary.ravetools-freqz2")
S3method(plot,"ravetools-freqz2")
S3method(plot,"ravetools-pwelch")
S3method(plot,"ravetools-wavelet-kernels")
S3method(plot,ravetools_crp)
S3method(plot,ravetools_curve)
S3method(print,"ravetools-multitaper-config")
S3method(print,"ravetools-printable")
S3method(print,"ravetools-pwelch")
S3method(print,"ravetools-wavelet-kernels")
S3method(print,ravetools_curve)
S3method(pwelch,default)
S3method(pwelch,matrix)
S3method(shift_array,array)
S3method(summary,"ravetools-freqz2")
export(as_matrix4)
export(as_quaternion)
export(as_vector3)
export(band_pass1)
export(band_pass2)
export(baseline_array)
export(blackman)
export(blackmanharris)
export(blackmannuttall)
export(bohmanwin)
export(buttap)
export(butter)
export(butter_max_order)
export(buttord)
export(carla)
export(catmull_rom_3d)
export(cheb1ap)
export(cheb1ord)
export(cheb2ap)
export(cheb2ord)
export(cheby1)
export(cheby2)
export(check_filter)
export(collapse)
export(convolve_image)
export(convolve_signal)
export(convolve_volume)
export(crp)
export(decimate)
export(design_filter)
export(design_filter_fir)
export(design_filter_iir)
export(detect_threads)
export(detrend)
export(diagnose_channel)
export(diagnose_filter)
export(dijkstras_surface_distance)
export(ellip)
export(ellipap)
export(ellipord)
export(ensure_mesh3d)
export(fast_cov)
export(fast_median)
export(fast_mvmedian)
export(fast_mvquantile)
export(fast_quantile)
export(fftw_c2c)
export(fftw_c2c_2d)
export(fftw_c2c_3d)
export(fftw_c2r)
export(fftw_r2c)
export(fftw_r2c_2d)
export(fftw_r2c_3d)
export(fill_surface)
export(filter_signal)
export(filtfilt)
export(find_peaks)
export(fir1)
export(firls)
export(flattopwin)
export(freqz2)
export(gammatone_fast)
export(grow_volume)
export(hamming)
export(hanning)
export(hilbert)
export(internal_rave_function)
export(is_not_cran)
export(matlab_palette)
export(mesh_from_volume)
export(morlet_wavelet)
export(multitaper)
export(multitaper_config)
export(mv_pwelch)
export(mvfftw_c2c)
export(mvfftw_c2r)
export(mvfftw_r2c)
export(naive_nmf)
export(new_matrix4)
export(new_quaternion)
export(new_vector3)
export(notch_filter)
export(plane_geometry)
export(plot_mesh_dotcloud)
export(plot_mesh_polygon)
export(plot_signals)
export(project_plane)
export(pwelch)
export(ravetools_threads)
export(raw_to_float)
export(raw_to_int16)
export(raw_to_int32)
export(raw_to_int64)
export(raw_to_int8)
export(raw_to_string)
export(raw_to_uint16)
export(raw_to_uint32)
export(raw_to_uint8)
export(rcond_filter_ar)
export(register_volume)
export(resample)
export(resample_3d_volume)
export(rgl_call)
export(rgl_plot_normals)
export(rgl_view)
export(shift_array)
export(stimpulse_align)
export(stimpulse_extract)
export(stimpulse_find)
export(stimpulse_interpolate)
export(surface_path)
export(vcg_isosurface)
export(vcg_kdtree_nearest)
export(vcg_mesh_volume)
export(vcg_raycaster)
export(vcg_smooth_explicit)
export(vcg_smooth_implicit)
export(vcg_sphere)
export(vcg_subdivision)
export(vcg_subset_vertex)
export(vcg_uniform_remesh)
export(vcg_update_normals)
export(wavelet_cycles_suggest)
export(wavelet_kernels)
importFrom(R6,R6Class)
importFrom(R6,is.R6)
importFrom(Rcpp,sourceCpp)
importFrom(graphics,image)
importFrom(gsignal,buttap)
importFrom(gsignal,butter)
importFrom(gsignal,buttord)
importFrom(gsignal,cheb1ap)
importFrom(gsignal,cheb1ord)
importFrom(gsignal,cheb2ap)
importFrom(gsignal,cheb2ord)
importFrom(gsignal,cheby1)
importFrom(gsignal,cheby2)
importFrom(gsignal,ellip)
importFrom(gsignal,ellipap)
importFrom(gsignal,ellipord)
importFrom(gsignal,hilbert)
importFrom(gsignal,resample)
importFrom(stats,approx)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(stats,sd)
useDynLib(ravetools, .registration = TRUE)
