% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aaa.R
\name{matlab_palette}
\alias{matlab_palette}
\title{'Matlab' heat-map plot palette}
\usage{
matlab_palette()
}
\value{
vector of 64 colors
}
\description{
'Matlab' heat-map plot palette
}
